@@ -3,6 +3,7 @@
 Module for application that behaves as command line interface
 """
 import shutil
+import socket
 import sys
 import traceback
 from pathlib import Path
@@ -12,6 +13,7 @@ import argparse
 import cryptnoxpy
 
 try:
+    import config
     import enums
     from command.options import options
     from command.helper import security
@@ -21,6 +23,7 @@ try:
         TimeoutException
     )
 except ImportError:
+    from . import config
     from . import enums
     from .command.options import options
     from .command.helper import security
@@ -92,9 +95,6 @@ class ErrorParser(argparse.ArgumentParser):
             "-h,": "General"
         }
         message = super().format_help()
-        if not message.find("{"):
-            return message
-
         lines = message.split("\n")
         ErrorParser._remove_lines(lines)
 
@@ -107,15 +107,13 @@ class ErrorParser(argparse.ArgumentParser):
                 continue
             if skip:
                 continue
-            if not line:
-                continue
 
             command = line.strip().split(" ")[0]
             if command in groups:
                 lines_out += ["", f"{groups[command]}:", ""]
             lines_out.append(line)
 
-        message = "\n".join(lines_out) if lines_out else message
+        message = "\n".join(lines_out)
 
         return message
 
@@ -209,8 +207,9 @@ class InteractiveCli:
     Application running a command line interface.
 
     :param str version: Version of the application the interface should return
-    :param bool debug: Print out debug information in regards to the
-                       communication to the card
+    :param bool debug: Print out debug information in regard to the communication to the card
+    :param bool remote: Determines whether the application should use remote feature
+                        or look for local readers
     """
 
     class ExitException(Exception):
@@ -218,10 +217,10 @@ class InteractiveCli:
         Exception to handle when the user requests to exit the application.
         """
 
-    def __init__(self, version: str, card_type: int = 0, debug: bool = False):
+    def __init__(self, version: str, debug: bool = False, port: int = None):
         self.version = version
-        self.card_type = card_type
         self.debug: bool = debug
+        self.port: int = port
 
         self._reconnect = False
         self._card_info = None
@@ -263,23 +262,59 @@ class InteractiveCli:
 
         return 0
 
+    def _client(self):
+        if not self.port:
+            return None
+
+        print(f"Found port {self.port} defined, searching remotely.")
+        try:
+            client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            client.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+            server = socket.gethostbyname(socket.gethostname() + ".local")
+            client.connect((server, self.port))
+        except Exception as error:
+            client = None
+            print(f'Remote card connection not found: {error}')
+        else:
+            config.REMOTE_CONNECTIONS.append(client)
+
+        return client
+
+    def _close_client(self, client):
+        if not self.port or not client:
+            return
+
+        message = '!Close'.encode('utf-8')
+        msg_length = len(message)
+        send_length = str(msg_length).encode('utf-8')
+        send_length += b' ' * (64 - len(send_length))
+
+        try:
+            client.send(send_length)
+            client.send(message)
+            client.close()
+        except Exception as e:
+            print(f'Probably no socket to close: {e}')
+
     def _run(self):
         print("Loading cards...")
+        client = self._client()
+
         try:
-            self._cards.refresh()
+            self._cards.refresh(self.port and client is not None)
             self._card_info = list(self._cards.values())[0].info
         except IndexError:
             pass
 
         self._cards.print_card_list(show_warnings=True, print_with_one_card=True)
         print("\n\nType help for list of commands.\n\n")
-        print("\n\nWith any input that is requested from you, to exit the current command "
-              "type: exit \n\n")
+        print("\n\nWith any input that is requested from you, to exit the current command type: exit \n\n")
 
         while True:
             try:
                 self._process_command()
             except InteractiveCli.ExitException:
+                self._close_client(client)
                 break
 
     def is_valid_subcommand(self, new_subcommand: List[str],
@@ -400,7 +435,7 @@ class InteractiveCli:
 
         if not self._card_info and not always_run:
             try:
-                self._cards.refresh()
+                self._cards.refresh(self.port != None)
                 self._card_info = list(self._cards.values())[0].info
             except IndexError:
                 print("No cards found.\n")
@@ -412,13 +447,12 @@ class InteractiveCli:
         if execute_result == -1:
             return
 
-        if not always_run:
-            try:
-                self._card_info = self._cards[command.serial_number].info
-            except KeyError:
-                pass
-            except (cryptnoxpy.CryptnoxException, ExitException, TimeoutException) as error:
-                print(error)
+        try:
+            self._card_info = self._cards[command.serial_number].info
+        except KeyError:
+            pass
+        except (cryptnoxpy.CryptnoxException, ExitException, TimeoutException) as error:
+            print(error)
 
         print("\n")
 
